import subprocess
import os


def subprocess_args(include_stdout=True):
    if hasattr(subprocess, 'STARTUPINFO'):
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        env = os.environ
    else:
        si = None
        env = None
    if include_stdout:
        ret = {'stdout': subprocess.PIPE}
    else:
        ret = {}
    ret.update({'stdin': subprocess.PIPE,
                'stderr': subprocess.PIPE,
                'startupinfo': si,
                'env': env})
    return ret



from typing import Union


class EmulatorOptions:
    """
    options:
    - resolution: width, height, DPI
    - cpu: 1|2|3|4
    - memory: M256|M521|M768|M1024|M1536|M2048|M3078|M4096|M6144
    - manufacturer: samsung, asus, ...
    - model: SM-G970N, ASUS_Z00DUO, ...
    - number: phone number
    - imei: auto
    - imsi: auto
    - simserial: auto
    - android id: auto
    - mac: auto
    - autorotate: True|False
    - lockwindow: True|False
    """
    M256 = 256
    M521 = 521
    M768 = 768
    M1024 = 1024
    M1536 = 1536
    M2048 = 2048
    M3072 = 3072
    M4096 = 4096
    M6144 = 6144
    MEMORY_OPTIONS = [M256, M521, M768, M1024, M1536, M2048, M3072, M4096, M6144]

    def __init__(self) -> None:
        self._options = {}
        self._resolution = (None, None, None)
        self._cpu = None
        self._memory = None
        self._manufacturer = None
        self._model = None
        self._phone_number = None
        self._imei = None
        self._imsi = None
        self._android_id = None
        self._mac = None
        self._auto_rotate = None
        self._lock_window = None

    @property
    def resolution(self):
        return self._resolution

    @resolution.setter
    def resolution(self, new_resolution: Union[list[int, int, int], tuple[int, int, int]]):
        if not isinstance(new_resolution, (list, tuple)):
            raise ValueError("Resolution must be list or tuple")
        if len(new_resolution) < 3:
            raise ValueError("Resolution need 3 arguments: width, height, dpi")
        self.set_resolution(*new_resolution[:3])

    def set_resolution(self, width: int, height: int, dpi: int):
        if width < 64:
            width = 64
        elif width > 4096:
            width = 4096
        if height < 64:
            height = 64
        elif height > 4096:
            height = 4096
        if dpi < 10:
            dpi = 10
        elif dpi > 640:
            dpi = 640
        self._resolution = (width, height, dpi)
        self._options["--resolution"] = f"{width},{height},{dpi}"

    @property
    def cpu(self):
        return self._cpu

    @cpu.setter
    def cpu(self, new_cpu: int):
        self.set_cpu(new_cpu)

    def set_cpu(self, core: int) -> None:
        if not isinstance(core, int):
            raise ValueError("CPU must be an integer between 1 and 4")
        if core < 1:
            core = 1
        elif core > 4:
            core = 4
        self._cpu = core
        self._options["--cpu"] = self._cpu

    @property
    def memory(self):
        return self._memory

    @memory.setter
    def memory(self, new_memory: int):
        self.set_memory(new_memory)

    def set_memory(self, byte: int) -> None:
        if byte not in self.MEMORY_OPTIONS:
            raise ValueError("byte must be one in " + str(self.MEMORY_OPTIONS))
        self._memory = byte
        self._options["--memory"] = self._memory

    @property
    def manufacturer(self):
        return self._manufacturer

    @manufacturer.setter
    def manufacturer(self, new_manufacturer: str):
        self.set_manufacturer(new_manufacturer)

    def set_manufacturer(self, manufacturer: str) -> None:
        if not isinstance(manufacturer, str):
            raise ValueError("Manufacturer must be a string")
        self._manufacturer = manufacturer
        self._options["--manufacturer"] = self._manufacturer

    @property
    def model(self):
        return self._model

    @model.setter
    def model(self, new_model: str):
        self.set_model(new_model)

    def set_model(self, model: str) -> None:
        if not isinstance(model, str):
            raise ValueError("Model must be a string")
        self._model = model
        self._options["--model"] = self._model

    @property
    def phone_number(self):
        return self._phone_number

    @phone_number.setter
    def phone_number(self, new_phone_number: str):
        self.set_phone_number(new_phone_number)

    def set_phone_number(self, phone_number: str) -> None:
        if not isinstance(phone_number, str):
            raise ValueError("PhoneNumber must be a string")
        self._phone_number = phone_number
        self._options["--pnumber"] = self._phone_number

    @property
    def imei(self):
        return self._imei

    @imei.setter
    def imei(self, new_imei: str):
        self.set_imei(new_imei)

    def set_imei(self, imei: str) -> None:
        if not isinstance(imei, str):
            raise ValueError("IMEI must be a string")
        self._imei = imei
        self._options["--imei"] = self._imei

    @property
    def imsi(self):
        return self._imsi

    @imsi.setter
    def imsi(self, new_imsi: str):
        self.set_imsi(new_imsi)

    def set_imsi(self, imsi: str) -> None:
        if not isinstance(imsi, str):
            raise TypeError("IMSI must be a string")
        self._imsi = imsi
        self._options["--imsi"] = self._imsi

    @property
    def android_id(self):
        return self._android_id

    @android_id.setter
    def android_id(self, new_android_id: str):
        self.set_android_id(new_android_id)

    def set_android_id(self, android_id: str) -> None:
        if not isinstance(android_id, str):
            raise TypeError("Android ID must be a string")
        self._android_id = android_id
        self._options["--androidid"] = self._android_id

    @property
    def mac(self):
        return self._mac

    @mac.setter
    def mac(self, new_mac: str):
        self.set_mac(new_mac)

    def set_mac(self, mac: str) -> None:
        if not isinstance(mac, str):
            raise TypeError("Mac ID must be a string")
        self._mac = mac
        self._options["--mac"] = self._mac

    @property
    def auto_rotate(self):
        return self._auto_rotate

    @auto_rotate.setter
    def auto_rotate(self, new_auto_rotate: bool):
        self.set_auto_rotate(new_auto_rotate)

    def set_auto_rotate(self, b: bool = True) -> None:
        if not isinstance(b, bool):
            raise ValueError("Auto-rotate must be a boolean")
        self._auto_rotate = b
        self._options["--autorotate"] = int(self._auto_rotate)

    @property
    def lock_window(self):
        return self._lock_window

    @lock_window.setter
    def lock_window(self, new_lock_window: bool):
        self.set_lock_window(new_lock_window)

    def set_lock_window(self, b: bool = True) -> None:
        if not isinstance(b, bool):
            raise ValueError("Lock Window must be a boolean")
        self._lock_window = b
        self._options["--lockwindow"] = int(self._lock_window)

    @property
    def options(self):
        return self._options



from __future__ import annotations
import os
import subprocess
from .args import subprocess_args
from .em_object import ObjectEmulator
from typing import Iterator, Optional, Union
import pkg_resources
import json

__version__ = pkg_resources.require("emulator-thw")[0].version


class LDPlayer:
    """
    Main cotroll all player of LDPlayer
    """
    def __init__(self, ldplayer_dir: str) -> None:
        self.error = ""
        self.ld_dir = os.path.normpath(ldplayer_dir)
        if not os.path.exists(self.ld_dir):
            raise Exception(f'The path: "{ldplayer_dir}" invalid!')
        ldconsole = os.path.join(self.ld_dir, "ldconsole.exe")
        dnconsole = os.path.join(self.ld_dir, "dnconsole.exe")
        if os.path.exists(ldconsole):
            self.controller = f'"{ldconsole}"'
        elif os.path.exists(dnconsole):
            self.controller = f'"{dnconsole}"'
        else:
            raise Exception("ldconsole.exe or dnconsole.exe not found")
        self.emulators = EmulatorContainer()
        for info in self.list_index_name():
            e = ObjectEmulator(self, info["index"], info["name"])
            self.set_ADB_debugging(e, True)
            self.emulators.update({info["index"]: e, info["name"]: e})
        self.adb = os.path.join(self.ld_dir, "adb.exe")
        if not os.path.isfile(self.adb):
            raise Exception("ADB not found!")
        os.system(f'"{self.adb}" start-server')

    def new(self, name: str = None) -> ObjectEmulator:
        exists = self.list_index()
        i = 0
        for _ in range(len(exists)):
            if i not in exists:
                break
            i += 1
        if not name:
            name = "LDPlayer-" + str(i)
        cmd = f'{self.controller} add --name "{name}"'
        self._run_cmd(cmd)
        em = ObjectEmulator(self, i, name)
        self.emulators.update({i: em, name: em})
        return em

    def copy(self, emulator: ObjectEmulator, as_name: str):
        cmd = f'{self.controller} copy --name "{as_name}" --from {emulator.index}'
        self.error = self._run_cmd(cmd)
        exists = self.list_index()
        i = 0
        for _ in range(len(exists)):
            if i not in exists:
                break
            i += 1
        em = ObjectEmulator(self, i, as_name)
        self.emulators.update({i: em, as_name: em})
        return em

    def remove(self, e: ObjectEmulator) -> bool:
        cmd = f'{self.controller} remove {e.this}'
        self.error = self._run_cmd(cmd)
        if self.error:
            return False
        self.emulators.clear()
        for info in self.list_index_name():
            obj = ObjectEmulator(self, info["index"], info["name"])
            self.emulators.update({info["index"]: obj, info["name"]: obj})
        return True

    def list_name(self) -> list[str]:
        return self._run_cmd(f'{self.controller} list').replace("À×µçÄ£ÄâÆ÷", "LDPlayer").split("\r\n")[:-1]

    def list_index(self) -> list[int]:
        return [int(args.split(",")[0]) for args in self._run_cmd(f'{self.controller} list2').split("\r\n")[:-1]]

    def list_index_name(self) -> list[dict]:
        lst = []
        for string in self._run_cmd(f'{self.controller} list2').split("\r\n")[:-1]:
            args = string.split(",")
            lst.append({"index": int(args[0]), "name": args[1].replace("À×µçÄ£ÄâÆ÷", "LDPlayer")})
        return lst

    def list_running(self) -> list[str]:
        return self._run_cmd(f'{self.controller} runninglist').split("\r\n")[:-1]

    def sort_window(self) -> bool:
        cmd = f'{self.controller} sortWnd'
        self.error = self._run_cmd(cmd)
        return False if self.error else True

    def setting(
        self,
        fps: int = None,
        audio: bool = None,
        fastplay: bool = None,
        cleanmode: bool = None
    ) -> bool:
        cmd = f'{self.controller} globalsetting'
        if fps is not None:
            if fps > 60:
                fps = 60
            elif fps < 1:
                fps = 1
            cmd += ' --fps ' + str(fps)
        if audio is not None:
            cmd += ' --audio ' + str(int(audio))
        if fastplay is not None:
            cmd += ' --fastplay ' + str(int(fastplay))
        if cleanmode is not None:
            cmd += ' --cleanmode ' + str(int(cleanmode))
        self.error = self._run_cmd(cmd)
        return False if self.error else True

    def quit_all(self) -> bool:
        cmd = f'{self.controller} quitall'
        self.error = self._run_cmd(cmd)
        return False if self.error else True

    def exit(self) -> bool:
        return os.system(f'"{self.adb}" kill-server')

    def set_ADB_debugging(self, em_obj: ObjectEmulator, is_on: bool) -> None:
        # Some LDPlayer version ADB debugging turn on but cannot connect
        config_file = os.path.join(self.ld_dir, "vms", "config", "leidian" + str(em_obj.index) + ".config")
        with open(config_file, "r") as f:
            config = json.load(f)
        config["basicSettings.adbDebug"] = int(is_on)
        with open(config_file, "w") as f:
            json.dump(config, f)

    def _run_cmd(self, cmd: str, decode: Optional[str] = 'latin-1') -> Union[str, bytes]:
        p = subprocess.Popen(cmd, **subprocess_args())
        o, e = p.communicate()
        if p.wait():
            return e.decode(decode) if decode is not None else e
        return o.decode(decode) if decode is not None else o

    def __enter__(self):
        return self

    def __exit__(self, _exc_type, _exc_value, traceback):
        self.quit_all()
        self.kill()
        if traceback:
            print(traceback)


class EmulatorContainer(dict):

    def __getitem__(self, __k) -> ObjectEmulator:
        return super().__getitem__(__k)

    def __iter__(self) -> Iterator[ObjectEmulator]:
        return iter([self[k] for k in self.keys() if type(k) is int])

    def __len__(self) -> int:
        return len([x for x in self.keys() if type(x) is int])

    def __str__(self) -> str:
        return "Emulators(" + ", ".join([self[k].__str__() for k in self.keys() if type(k) is int]) + ")"



import cv2
import numpy as np


def get_pos_img(
    obj: str,
    _in: bytes,
    center: bool = True,
    multi: bool = False,
    threshold: float = 0.8,
    eps: float = 0.05,
    show: bool = False
) -> list:
    try:
        img_base = cv2.imdecode(np.frombuffer(_in, dtype=np.uint8), flags=cv2.IMREAD_COLOR)
        tem = cv2.imread(obj)
        height, width = tem.shape[:2]
        result = cv2.matchTemplate(img_base, tem, cv2.TM_CCOEFF_NORMED)
        pos = []
        if multi:
            y_loc, x_loc = np.where(result >= threshold)
            rectangles = []
            for x, y in zip(x_loc, y_loc):
                rectangles.append([int(x), int(y), int(width), int(height)])
                rectangles.append([int(x), int(y), int(width), int(height)])
            for x, y, w, h in cv2.groupRectangles(rectangles, 1, eps)[0]:
                if show:
                    cv2.rectangle(img_base, (x, y), (x + w, y + h), (0, 0, 255), 2)
                if center:
                    pos.append((x + w // 2, y + h // 2))
                else:
                    pos.append((x, y))
        else:
            _, max_val, _, max_loc = cv2.minMaxLoc(result)
            if max_val > threshold:
                if show:
                    cv2.rectangle(img_base, max_loc, (max_loc[0] + width, max_loc[1] + height), (0, 0, 255), 2)
                if center:
                    pos.append((max_loc[0] + width // 2, max_loc[1] + height // 2))
                else:
                    pos.append(max_loc)
        if show:
            cv2.imshow("show", img_base)
            cv2.waitKey(0)
            cv2.destroyAllWindows()
        return pos
    except Exception as e:
        print(e)
        return []



from setuptools import setup
import pathlib
import shutil
import sys
import os


here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

if sys.argv[-1] == 'update':
    if os.path.isdir('dist'):
        shutil.rmtree('dist')
    if os.path.isdir('emulator_Thw.egg-info'):
        shutil.rmtree('emulator_Thw.egg-info')
    os.system('python -m build')
    os.system('twine upload dist/*')
    sys.exit()


setup(
    name='emulator-Thw',
    version='0.0.9',
    description='package help control emulator',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Thachj-Thw/emulator',
    author='Thw',
    author_email='',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Natural Language :: Vietnamese'
    ],
    keywords='emulator, android, android emulator',
    packages=['emulator'],
    install_requires=['numpy', 'opencv-python'],
    python_requires='>=3.6',
    include_package_data=True,
    project_urls={
        'Source': 'https://github.com/Thachj-Thw/emulator',
    },
)
